lexer grammar fsqLexer;
  
options {
	language = Java;
}

@header {
  package de.uni_tuebingen.sfb.lichtenstein.formulas.parsing;
}

/**
 * Opening paren, starts each formula.
 */
OPEN : '(' ;

/**
 * Closing paren, ends each formula.
 */
CLOSE : ')' ;

/**
 * labels
 */
FUNCTION : 'fct' ;
CATEGORY : 'cat' ;
WORD : 'word' ;
LEMMA : 'lemma' ;
MORPHOLOGY : 'morph' ;

SENTENCE : 'sent' ;

/**
 * atomic
 */
EQUALITY : '=' ;
PROPER_DOMINANCE : '>+' ;
IMMEDIATE_DOMINANCE : '>' ;
DOMINANCE : '>>' ;
IMMEDIATE_PRECEDENCE : '.' ;
PRECEDENCE : '..' ;
CONTAINMENT : 'in' ;

/**
 * unary
 */
NEGATION : '!' ;

/**
 * binary
 */
IMPLICATION : '->' ;
EQUIVALENCE : '<->' ;

/**
 * n-ary
 */
DISJUNCTION : '|' ;
CONJUNCTION : '&' ;

/**
 * Quantifications.
 */
SECOND_ORDER_UNIVERSAL : 'A2' ;
SECOND_ORDER_EXISTENTIAL : 'E2' ;
FIRST_ORDER_UNIVERSAL : 'A1' ;
FIRST_ORDER_EXISTENTIAL : 'E1' ;

/**
 * Lisp-like line comment, starts with ';', can be ignored.
 */
LINE_COMMENT : ';' ~('\n'|'\r')* '\r'? '\n' { $channel = HIDDEN; } ;

/**
 * A new line on any platform?  Can be ignored.
 */
NEWLINE
	: ( '\r\n' | '\r' | '\n' )
		{ $channel = HIDDEN; }
	;

/**
 * Whitespace symbols.
 */
fragment WS : ( ' ' | '\t' ) ;

/**
 * Whitespace; should not be ignored, acts as delimiter.
 */
WHITESPACE : WS+ ;

/**
 * An identifier can be anything except parens or whitespace.  If people want parens, we are in trouble.
 */
IDENTIFIER
	: ( ~( CLOSE | OPEN | WS ) )+
	;
